﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using Infragistics.Controls.Grids;

namespace GridsCell
{
    public partial class MainWindow : Window
    {
        private readonly ObservableCollection<Data> collection;

        public MainWindow()
        {
            InitializeComponent();
            collection = new ObservableCollection<Data>();
            xamGrid.ItemsSource = collection;
        }

        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            collection.Add(new Data {Name = "1", Age = 20, MaxAge = 40});
            collection.Add(new Data { Name = "3", Age = 70, MaxAge = 25});
            collection.Add(new Data { Name = "4", Age = 50, MaxAge = 40 });
        }

        private void ApplyLayout(object sender, RoutedEventArgs e)
        {
            xamGrid.Columns.Add(new TextColumn() { Key = "Name", HeaderText = "Name" });
            xamGrid.Columns.Add(new TextColumn() { Key = "Age", HeaderText = "Age" });
            xamGrid.Columns.Add(new TextColumn() { Key = "MaxAge", HeaderText = "MaxAge" });
        }

        private void AddFormatting(object sender, RoutedEventArgs e)
        {
            xamGrid.ConditionalFormattingSettings.AllowConditionalFormatting = true;

            var stateTextColumn = xamGrid.Columns["Age"] as TextColumn;
            if (stateTextColumn != null)
            {
                var style = TryFindResource("ConditionalFormattingCellControlStyle") as Style;
                if (style == null) return;

                #region Issue No1 ... Why is the below style not applied? I am overriding Bar Percentage Property
                style.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.BarPercentageProperty, Value = 50.0 });
                stateTextColumn.CellStyle = style;
                stateTextColumn.ConditionalFormatCollection.Add(new DataBarConditionalFormatRule());
                #endregion

                #region Issue No 2 -- Comment the above region and uncomment the below two lines
                //var higherBidConditionalRule = new HigherBidConditionalRule(style);
                //stateTextColumn.ConditionalFormatCollection.Add(higherBidConditionalRule);
                #endregion
            }
        }
    }

    public class Data
    {
        public String Name { get; set; }
        public Double Age { get; set; }
        public Double MaxAge { get; set; }
    }

    public class ConditionalFormatRule : ConditionalFormattingRuleBaseProxy
    {
        protected Style YellowStyle { get; set; }
        protected Style RedStyle { get; set; }

        public ConditionalFormatRule()
        {
            
            var yellowStyle = new Style(typeof(ConditionalFormattingCellControl));
            
            YellowStyle = yellowStyle;
        }


        protected override Style EvaluateCondition(object sourceDataObject, object sourceDataValue)
        {
            return null;
        }
    }


    public class HigherBidConditionalRule : ConditionalFormattingRuleBase
    {
        private readonly Style style;
        public HigherBidConditionalRule(Style cellStyle)
        {
            style = cellStyle;
        }

        protected override IConditionalFormattingRuleProxy CreateProxy()
        {
            return new HigherBidConditionRuleProxy(style);
        }

    }

    public class HigherBidConditionRuleProxy : ConditionalFormatRule
    {
        private readonly Style style;

        public HigherBidConditionRuleProxy(Style style)
        {
            this.style = style;
        }

        protected override Style EvaluateCondition(object sourceDataObject, object sourceDataValue)
        {
            Double val;
            if (!Double.TryParse(sourceDataValue.ToString(), out val))
            {
                return null;
            }

            var data = sourceDataObject as Data;
            if (data != null)
            {
                if (val > data.MaxAge)
                {
                    var relative = (val/data.MaxAge) * 100;
                    style.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.BarPercentageProperty, Value = relative });
                    return style;
                }
            }
            return null;
        }
    }
   }
